@@ -4,29 +4,22 @@ from .device import Device
 from .utils import notNone
 from .const import (
     #LOGGER,
-    STATUS_RESPONSE_METERS,
-    STATUS_RESPONSE_EMETERS,
-    STATUS_RESPONSE_METERS_POWER,
-    STATUS_RESPONSE_METERS_TOTAL,
-    STATUS_RESPONSE_METERS_VOLTAGE,
-    STATUS_RESPONSE_METERS_PF,
-    STATUS_RESPONSE_METERS_CURRENT,
-    STATUS_RESPONSE_METERS_TOTAL_RETURNED,
-    INFO_VALUE_CURRENT_CONSUMPTION,
+    ATTR_RPC,
     INFO_VALUE_TOTAL_CONSUMPTION,
     INFO_VALUE_TOTAL_RETURNED,
-    INFO_VALUE_VOLTAGE,
     INFO_VALUE_POWER_FACTOR,
     INFO_VALUE_CURRENT,
+    INFO_VALUE_VOLTAGE,
     ATTR_POS,
     ATTR_PATH,
-    ATTR_FMT
+    ATTR_FMT,
+    ATTR_TOPIC
 )
 
 class PowerMeter(Device):
     """Class to represent a power meter value"""
     def __init__(self, block, channel, position = None,
-                 tot_pos = None, voltage_to_block=False, em=False):
+                 tot_pos = None, voltage_to_block=False, em=False, topic="emeter"):
         #Todo: voltage_to_block
         super(PowerMeter, self).__init__(block)
         self.id = block.id
@@ -36,6 +29,9 @@ class PowerMeter(Device):
             self.device_nr = channel
         else:
             self._channel = 0
+        self.is_device = False
+        self.is_sensor = True
+
         # if meters is None:
         #     self.meters = [self._channel]
         # else:
@@ -44,41 +40,52 @@ class PowerMeter(Device):
         self.device_type = "POWERMETER"
         self.info_values = {}
         self.state = None
+        meters = "emeters" if em else "meters"
         self._state_cfg = {
             ATTR_POS: notNone(position, [111, 4101, 4102, 4105]),
-            ATTR_PATH: 'meters/$/power',
-            ATTR_FMT: ['float']
+            ATTR_PATH: meters + '/$/power',
+            ATTR_FMT: ['float'],
+            ATTR_TOPIC: ['relay/$/power', topic + '/$/power'],
+            ATTR_RPC: 'switch:$/apower'
         }
-        meters = "emeters" if em else "meters"
         divider = None if em else '/60'
         self._info_value_cfg = {
-            INFO_VALUE_VOLTAGE : {
-                ATTR_POS: [116, 4108],
-                ATTR_PATH: meters + '/$/voltage',
-                ATTR_FMT: ['float']
-            },
             INFO_VALUE_POWER_FACTOR : {
                 ATTR_POS: [114, 4110],
-                ATTR_PATH: meters + 'eters/$/pf',
-                ATTR_FMT: ['float']
+                ATTR_PATH: meters + '/$/pf',
+                ATTR_FMT: ['float'],
+                ATTR_TOPIC: topic + '/$/pf'
             },
             INFO_VALUE_CURRENT : {
                 ATTR_POS: [115, 4109],
                 ATTR_PATH: meters + '/$/current',
-                ATTR_FMT: ['float']
+                ATTR_FMT: ['float'],
+                ATTR_TOPIC: topic + '/$/current',
+                ATTR_RPC: 'switch:$/current'
             },
             INFO_VALUE_TOTAL_CONSUMPTION : {
                 ATTR_POS: tot_pos or [4103, 4104, 4106],
                 ATTR_PATH: meters + '/$/total',
-                ATTR_FMT: ['float', divider, 'round:2']
+                ATTR_FMT: ['float', divider, 'round:2'],
+                ATTR_TOPIC: ['relay/$/energy', topic + '/$/total', topic + '/$/energy'],
+                ATTR_RPC: 'switch:$/aenergy/total'
             },
             INFO_VALUE_TOTAL_RETURNED :
             {
                 ATTR_POS: [4107],
                 ATTR_PATH: meters + '/$/total_returned',
-                ATTR_FMT: ['float', divider, 'round:2']
+                ATTR_FMT: ['float', divider, 'round:2'],
+                ATTR_TOPIC: topic + '/$/total_returned'
             }
         }
+        (self.block if voltage_to_block else self) \
+            ._info_value_cfg[INFO_VALUE_VOLTAGE] = \
+            {
+                ATTR_POS: [116, 4108],
+                ATTR_PATH: meters + '/$/voltage',
+                ATTR_FMT: ['float'],
+                ATTR_TOPIC: topic + '/$/voltage'
+            }
 
     # """
     # def update_status_information(self, status):
